# Helper module for configuring the VSRS software to run experiments
__author__ = "Beerend Ceulemans"
__email__ = "bceulema@etro.vub.ac.be"

# Path to the VSRS executable
VSRS_PATH    = 'D://FTV-test//VSRS//ViewSynVC11.exe'

# Uncompressed video+depth are assumed to be in VIDEO_FOLDER
# Filenames start from cam1.yuv and depth1.yuv and videos are in YUV420 format
# Decoded sequences are assumed to be in VIDEO_FOLDER-decoded/QP<insert_QP>
# Filenames of the decoded bitstreams are dec_0.yuv -> dec_x.yuv as used
# by the 3D-HEVC software. Even numbers are texture their subsequent odd number
# is the corresponding depth map. Take care that in the HEVC software, the view 
# with number 0 is the MIDDLE view, so renumbering is required cfr. the 
# VSRS_TO_HTM function below
SEQUENCE_NAME = 'Arc1'
VIDEO_FOLDER  = 'Arc1'
PARAM_FILE    = 'Arc1//Arc1_virtual_views_v4.1_equally_spaced.txt'
VIDEO_W      = 1920
VIDEO_H      = 1080

Z_NEAR       = [0,26.90196,26.27451,26.90196,25.01961,25.64706,25.96078,26.27451]
Z_FAR        = [0,100,100,100,100,100,100,100]

USE_UNCOMPRESSED_DEPTH = False

# return the folder of an input video for a given QP
def video_folder(QP):
    if QP == 0:
        return VIDEO_FOLDER
    else:
        return VIDEO_FOLDER + '-decoded\\\\QP' + str(QP)

# return the folder of an output virtual view for a given QP
def virtual_video_folder(QP):
    name = video_folder(QP) + '\\\\virtual'
    if USE_UNCOMPRESSED_DEPTH:
        name += '-origDepth'
    return name

# renumbering from VSRS to HTM
def VSRS_TO_HTM(camNr):
    mapping = [0, 7, 5, 3, 1, 9, 11, 13]
    return mapping[camNr]

# read the cameras in a parameter file
# the cameras are returned as a list of (name,x-coordinate) tuples
def readCameras():
    f = open(PARAM_FILE,'r')
    cameras   = []
    positions = []
    cam_name  = ''
    cam_x     = 0
    count     = 0
    for line in f:
        line = line.strip()
        if len(line)==0: # skip empty lines
            continue
        if line.startswith('param'): # if the line indicated a new camera
            cam_name = line
            count    = 0
            continue
        count += 1
        if count == 6: # skip to the line where we can find the x coordinate of the camera
            nums  = line.split()
            cam_x = float(nums[3])
            if not(cam_x in positions): # don't add cameras with the same x coordinates
                positions.append(cam_x)
                cameras.append((cam_name,cam_x))
            
    f.close()
    return cameras

# for a virtual camera, find a left-right pair in the orig_cams list
def findLeftRight(virtual,orig_cams):
    left_cam = None
    right_cam = None
    for orig_view in orig_cams:
        if orig_view[1] < virtual[1]:
            left_cam  = orig_view
        if orig_view[1] > virtual[1]:
            right_cam = orig_view
            break
    return left_cam,right_cam

# return the texture and depth files of the input virtual camera for a given QP
def colorAndDepthFileNames(camera,QP):
    num  = int(camera[0].split('_')[-1])
    folder = video_folder(QP)
    if QP != 0:
        color = folder + '\\\\dec_' + str(VSRS_TO_HTM(num)-1) + '.yuv'
        depth = folder + '\\\\dec_' + str(VSRS_TO_HTM(num)) + '.yuv'
    else:
        color = folder + '\\\\cam' + str(num) + '.yuv'
        depth = folder + '\\\\depth' + str(num) + '.yuv'        
    return color,depth

# return the texture file of the input virtual camera for a given QP
def colorFileName(camera,QP):
    num  = int(camera[0].split('_')[-1])
    folder = video_folder(QP)
    if QP != 0:
        color = folder + '\\\\dec_' + str(VSRS_TO_HTM(num)-1) + '.yuv'
    else:
        color = folder + '\\\\cam' + str(num) + '.yuv'      
    return color

# return the name of the output virtual camera for a left-right pair and QP
def virtualFileName(left,virtual,right,QP):
    num_left  = int(left[0].split('_')[-1])
    num_right = int(right[0].split('_')[-1])
    num_synth = virtual[0].split('_')[-1]
    folder = virtual_video_folder(QP)
    color_synth = folder + '\\\\virtual_' + num_synth + '_from_' + str(num_left) + '_' + str(num_right) + '.yuv'
    return color_synth

# create a config file for VSRS and return the name of the output yuv-file
def make_config(fileName,left,virtual,right,QP,start=0,nrOfFrames=1,temp=False,blend=0,precision=4,DepthType=1,origDepth=False):
    num_left  = int(left[0].split('_')[-1])
    num_right = int(right[0].split('_')[-1])    
    color_left,depth_left   = colorAndDepthFileNames(left,QP)
    color_right,depth_right =colorAndDepthFileNames(right,QP)
    if USE_UNCOMPRESSED_DEPTH == True or origDepth == True:
        tmp,depth_left  = colorAndDepthFileNames(left,0)
        tmp,depth_right = colorAndDepthFileNames(right,0)        
    if temp == True:
        color_synth = video_folder(QP) + '\\\\tmp.yuv'
    else:
        color_synth = virtualFileName(left,virtual,right,QP)
    config = 'DepthType ' + str(DepthType) + '\nSourceWidth ' + str(VIDEO_W) + '\nSourceHeight ' + str(VIDEO_H) + '\nStartFrame ' + str(start) + '\nTotalNumberOfFrames '+ str(nrOfFrames) + '\nLeftNearestDepthValue ' + str(Z_NEAR[num_left]) + '\nLeftFarthestDepthValue ' + str(Z_FAR[num_left]) + '\nRightNearestDepthValue ' + str(Z_NEAR[num_right]) + '\nRightFarthestDepthValue ' + str(Z_FAR[num_right]) + '\nCameraParameterFile ' + PARAM_FILE + '\nLeftCameraName ' + left[0] + '\nVirtualCameraName ' + virtual[0] + '\nRightCameraName ' + right[0] + '\nLeftViewImageName ' + color_left + '\nRightViewImageName ' + color_right + '\nLeftDepthMapName ' + depth_left + '\nRightDepthMapName ' + depth_right + '\nOutputVirtualViewImageName ' + color_synth + '\nColorSpace 0\nPrecision ' + str(precision) + '\nFilter 1\nSynthesisMode 0\nBoundaryNoiseRemoval 0\nViewBlending ' + str(blend)
    f_out = open(fileName,'w')
    f_out.write(config)
    f_out.close()
    return color_synth